------------------------------------- ELVUI IMPORTS ------------------------------------------------
local E, L, V, P, G, _ = unpack(ElvUI); --Import: Engine, Locales, PrivateDB, ProfileDB, GlobalDB
local SM = E:NewModule('SaveMe', 'AceHook-3.0', 'AceEvent-3.0', 'AceTimer-3.0')
-- SM.version = GetAddOnMetadata("SaveMe", "Version")

local UF = E:GetModule('UnitFrames')
local _, ns = ...
local ElvUF = ns.oUF
----------------------------------------------------------------------------------------------------

------------------------------------ GLOBALS AND STUFF ---------------------------------------------
function dbg_print(str)
	if E.private.SaveMe.debugPrint then
		print(str);
	end
end	
----------------------------------------------------------------------------------------------------

------------------------------------ SPELL LOOKUP TABLE --------------------------------------------
local ExternalBuffs = 
{
	["Druid"] = 
	{
		["Ironbark"] = 102342,
		["Cenarion_Ward"] = 102351,
	},
	["Hunter"] = 
	{
		["Roar_of_Sacrifice"] = 53480,
	},
	["Monk"] = 
	{
		["Life_Cocoon"] = 116849,
	},
	["Paladin"] = 
	{
		["Hand_of_Protection"] = 1022,
		["Hand_of_Sacrifice"] = 6940,
		["Hand_of_Purity"] = 114039,
		["Sacred_Shield"] = 20925,
		["Divine_Shield"] = 642,
	},
	["Priest"] = 
	{
		["Pain_Suppression"] = 33206,
		["Guardian_Spirit"] = 47788,
	},
	["Shaman"] = 
	{
		["Grounding_Totem"] = 8177,
	},
	["Warrior"] = 
	{
		["Intervene"] = 3411,
		["Safeguard"] = 114029,
		["Vigilance"] = 114030,
	},
};

function IsExternal(spellID, spellName)
	dbg_print("Checking if " .. spellName .. "(" .. spellID .. ") is an external.");
	for class, spells in pairs(ExternalBuffs) do
		--dbg_print(" - " .. class);
		for name, id in pairs(spells) do
			--dbg_print(" ---- " .. name .. "(" .. id .. ")");
			if id == spellID then
				dbg_print("It is");
				return true;
			end
		end
	end
	dbg_print("It isnt");
	return false;
end

----------------------------------------------------------------------------------------------------

------------------------------------ LOCAL VARIABLES -----------------------------------------------
local PlayerName;
local PlayerRealm;
local GlowList = {};
----------------------------------------------------------------------------------------------------

-------------------------------------- FUNCTIONS ---------------------------------------------------

function CropMyServer(playerName)
	dbg_print("Cropping " .. PlayerRealm .. " from " .. playerName .. " ...");
	local serverIndex = string.find(playerName, PlayerRealm);
	if serverIndex ~= nil then
		return string.sub(playerName, 1, serverIndex - 2); -- -2 to include the hyphen between the name and server
	end
	return playerName;
end

function SM:GetUnitFrame(targetUnit)

	local frameHeader, playerFrames, frame, _;
	frameHeader = self:GetGroupFrameHeader();
	
	-- find our player in our group
	if frameHeader then
		playerFrames = frameHeader:GetChildren();		
		for _, frame in ipairs(playerFrames) do
			-- check for matching names
			if frame.unit == targetUnit then
				return frame;
			end;
		end
	end	
	return nil;	
end

function SM:GetPlayerFrame(targetPlayer)

	local frameHeader, playerFrames, frame, _;
	frameHeader = self:GetGroupFrameHeader();
	
	-- find our player in our group
	if frameHeader then
		playerFrames = frameHeader:GetChildren();		
		for _, frame in ipairs(playerFrames) do
			-- check for matching names
			frameUnitName = GetUnitName(frame.unit, true);
			if frameUnitName == targetPlayer then
				dbg_print(frameUnitName .. " is " .. frame.unit);
				return frame;
			end;
		end
	end	
	return nil;	
end

function SM:ToggleEnabled(enabled)
	if enabled then
		dbg_print("SaveMe Enabled!")
	else
		dbg_print("SaveMe Disabled!")
	end
end

function SM:TogglePingSelf(pingSelf)
	if pingSelf then
		dbg_print("PingSelf Enabled!")
	else
		dbg_print("PingSelf Disabled!")
	end
end

function SM:PingTarget(targetUnit)

	-- dont ping ourselves if requested
	if E.private.SaveMe.pingSelf == false and senderNameShort == PlayerName then
		dbg_print("Ignoring my ping.");
		return;
	end
	
	-- start a ping glow
	targetFrame = SM:GetPlayerFrame(targetUnit);
	if targetFrame ~= nil then
	
		-- add to the list
		newGlow = Glow:New();
		newGlow.FrameTarget = targetFrame;
		newGlow:StartGlow();
		table.insert(GlowList, {targetUnit, newGlow});
		dbg_print("Pinging " .. targetUnit .. " for a Save!");
	end
end

function SM:BroadcastPing(targetPlayer)

	-- check if we are in a party or raid
	local FrameHeader = SM:GetGroupFrameHeader();
	if FrameHeader == nil then
		dbg_print("You are not in party or raid!");
		return;
	end
	
	-- check if the target is in our party
	local playerFrame = SM:GetPlayerFrame(targetPlayer);
	if playerFrame == nil then
		dbg_print(targetPlayer .. " isnt in your party or raid!");
		return;
	end

	-- filter out spaces 
	targetPlayer = string.gsub(targetPlayer, " ", "");
	
	-- ensure we are broadcasting the server name on the player
	if string.find(targetPlayer, "-") == nil then
		targetPlayer = targetPlayer .. "-" .. PlayerRealm;
	end

	-- broadcast out our ping
	SendAddonMessage(SM.name, targetPlayer, "RAID");
	
end

function SM:HandleCombatlog(...)
	local arg = {...};
	if arg[2] == "SPELL_AURA_APPLIED" then
		-- dbg_print(string.join(" | ", tostringall(...)));
		local castPlayer = arg[5];
		local castTarget = arg[9];
		local spellID = arg[12];
		local spellName = arg[13];
		local croppedTarget = CropMyServer(castTarget);
		
		-- check if the spell is an external buff
		local isExternalBuff = IsExternal(spellID, spellName);
		if isExternalBuff then
			-- check if the cast target is glowing
			for index, value in ipairs(GlowList) do
				local unitName = value[1];
				local unitGlow = value[2];
				
				-- if cast target is glowing, stop him from glowing		
				if unitName == croppedTarget then
					unitGlow:StopGlow();
					table.remove(GlowList, index);
					dbg_print(CropMyServer(castPlayer) .. " applied " .. spellName .. "(" .. spellID .. ") to " .. croppedTarget);
					break;
				end			
			end
		end
	end
end

function SM:GetGroupFrameHeader()
	FrameHeader = nil;
	if ElvUF_Party:IsShown() then
		FrameHeader = ElvUF_Party
	elseif ElvUF_Raid:IsShown() then
		FrameHeader = ElvUF_Raid
	elseif ElvUF_Raid40:IsShown() then
		FrameHeader = ElvUF_Raid40
	end
	return FrameHeader;
end

function SM:Initialize()
	PlayerName = GetUnitName("player", true);
	PlayerRealm = GetRealmName();
	
	-- remove spaces from the realm name (e.g. for Kul Tiras)
	PlayerRealm = string.gsub(PlayerRealm, " ", "");
	
	dbg_print("SaveMe Initialized!  use " .. SLASH_SAVEME2 .. "; " .. SLASH_SAVEME1 .. "; or " .. SLASH_SAVEME1 .. " [Player Name] to ping for help!")
end
----------------------------------------------------------------------------------------------------

---------------------------------- SAVEME FRAME INFO -----------------------------------------------
local SaveMeFrame = CreateFrame("Frame");
function SaveMeFrame:OnUpdate(deltaTime)
		
	-- foreach active ping
	for index, value in ipairs(GlowList) do
	
		local unitName = value[1];
		local unitGlow = value[2];
		
		-- update
		unitGlow:UpdateGlow(deltaTime);
		
		-- remove ping if needed
		if unitGlow.GlowTime < 0.0 then
			unitGlow:StopGlow();
			table.remove(GlowList, index);
		end		
	end
		
end

function SaveMeFrame:EventHandler(event, ...)
	-- dbg_print(event);
	if event == "CHAT_MSG_ADDON" then
		local prefix, targetUnit, channel, senderNameFull, senderNameShort = ...;
		if prefix == SM.name then
			dbg_print(prefix .. ": |" .. targetUnit .. "| |" .. channel .. "| |" .. senderNameFull .. "| |" .. senderNameShort .. "|");
			
			-- get hte players name if they are on the same server
			targetUnit = CropMyServer(targetUnit);
					
			-- ping the target
			SM:PingTarget(targetUnit);		
		end
	-- elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
		SM:HandleCombatlog(...);			
	else
		-- dbg_print("Unhandled Event: " .. event);
	end
end
RegisterAddonMessagePrefix(SM.name);
SaveMeFrame:RegisterEvent("CHAT_MSG_ADDON");
SaveMeFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
SaveMeFrame:SetScript("OnUpdate", SaveMeFrame.OnUpdate);
SaveMeFrame:SetScript("OnEvent", SaveMeFrame.EventHandler);

----------------------------------------------------------------------------------------------------

-------------------------------------- SLASH COMMAND -----------------------------------------------
SLASH_SAVEME1 = "/saveme";
SLASH_SAVEME2 = "/sm";
function SlashCmdList.SAVEME(msg, editbox)
	
	-- only throw the save request if enables
	if E.private.SaveMe.enabled == false then
		dbg_print("Please enable SaveMe in the ElvUI Settings (/elvui)");
	else
		-- get the player target to ping
		local targetPlayer; 
		
		---------- Handle various options ----------
		if msg == "TableDisplay" then
			for k,v in pairs(SM) do
				dbg_print("SM." .. k);
			end
			return;
		elseif msg == "group" then
			local GroupHeader = SM:GetGroupFrameHeader();
			
			if GroupHeader then
				local playerFrames = GroupHeader:GetChildren();		
				for k, v in ipairs(playerFrames) do
					dbg_print(GetUnitName(v.unit, true) .. " is " .. v.unit);
				end
			end	
			return;
		elseif msg == "print" then
			dbg_print(PlayerRealm);
			return;
		---------- Handle the actual targeting ----------
		elseif msg == "" then
			targetPlayer = PlayerName;
		elseif msg == "%t" then
			targetPlayer = GetUnitName("target", true);
		elseif msg == "%f" then
			targetPlayer = GetUnitName("focus", true);
		else
			targetPlayer = msg;
		end
		
		-- sanity check
		if targetPlayer == nil then
			dbg_print("Invalid Target!");
			return;
		end
		
		-- broadcast
		SM:BroadcastPing(targetPlayer);
	end
end
----------------------------------------------------------------------------------------------------


E:RegisterModule(SM:GetName())