local E, L, V, P, G, _ = unpack(ElvUI); --Import: Engine, Locales, PrivateDB, ProfileDB, GlobalDB, Localize Underscore
local RangeModifiersModule = E:GetModule('RangeModifiers', 'AceHook-3.0', 'AceEvent-3.0', 'AceTimer-3.0')

local UF = E:GetModule('UnitFrames')
local _, ns = ...
local ElvUF = ns.oUF

local f = CreateFrame("Frame")
f:RegisterEvent("GROUP_ROSTER_UPDATE")
f:SetScript("OnEvent", function()
	RangeModifiersModule:ClearUnitsRangeGlow()
	RangeModifiersModule:CreateUnitsRangeGlow()
end)

local f = CreateFrame("Frame");
function f:onUpdate(sinceLastUpdate)
	self.sinceLastUpdate = (self.sinceLastUpdate or 0) + sinceLastUpdate
	if (self.sinceLastUpdate >= E.private.rangeModifiers.update.interval) then
		RangeModifiersModule:UpdateAlpha()
		RangeModifiersModule:UpdateUnitsRange()
		
		self.sinceLastUpdate = 0;
	end
end
f:SetScript("OnUpdate", f.onUpdate)

local alpha = 0
local alphaDir = 1
local alphaValue = alpha

function RangeModifiersModule:UpdateAlpha()
	alpha = alpha + (E.private.rangeModifiers.alphaData.alphaIncrement * alphaDir)
	
	if alpha >= E.private.rangeModifiers.alphaData.maxAlpha then
		alpha = E.private.rangeModifiers.alphaData.maxAlpha
		alphaDir = -1
	end
	
	if alpha <= E.private.rangeModifiers.alphaData.minAlpha then
		alpha = E.private.rangeModifiers.alphaData.minAlpha
		alphaDir = 1
	end
	
	alphaValue = E.private.rangeModifiers.general.frameColour.a * alpha
end

function RangeModifiersModule:ToggleEnabled()
	if not E.private.rangeModifiers.enabled then		
		RangeModifiersModule:ClearUnitsRangeGlow()
		RangeModifiersModule:CreateUnitsRangeGlow()
		print(L["RANGEMODIFIERS_DISABLED"])
	else 
		RangeModifiersModule:ClearUnitsRangeGlow()
		print(L["RANGEMODIFIERS_ENABLED"])
	end
end

function RangeModifiersModule:ClearUnitsRangeGlow()
	local header, children, child, childCount
	
	childCount = 0
	
	if ElvUF_Party:IsShown() then
		header = ElvUF_Party
	elseif ElvUF_Raid:IsShown() then
		header = ElvUF_Raid
	elseif ElvUF_Raid40:IsShown() then
		header = ElvUF_Raid40
	end
	
	if header then	
		children = header:GetChildren()
		for _, child in ipairs(children) do		
			if child then
				childCount = childCount + 1
				
				if child.RangeGlow then
					if E.private.rangeModifiers.debug then
						print("clearing RangeGlow for : " .. child.unit)
					end
					
					child.RangeGlow:Hide()
					child.RangeGlow:SetParent(E.UIParent)
					child.RangeGlow:ClearAllPoints()
				end
			end
		end
	end
	
	if E.private.rangeModifiers.debug then
		print("clearing " .. childCount .. " RangeGlows")
	end
end

function RangeModifiersModule:CreateUnitsRangeGlow()
	local header, children, child, childCount
	
	childCount = 0
	
	if ElvUF_Party:IsShown() then
		header = ElvUF_Party
	elseif ElvUF_Raid:IsShown() then
		header = ElvUF_Raid
	elseif ElvUF_Raid40:IsShown() then
		header = ElvUF_Raid40
	end
	
	if header then	
		children = header:GetChildren()
		for _, child in ipairs(children) do		
			if child.unit and UnitIsPlayer(child.unit) then
				childCount = childCount + 1
				
				if E.private.rangeModifiers.debug then
					print("creating RangeGlow for : " .. child.unit)
				end
				
				child.RangeGlow = child:CreateTexture(nil, "OVERLAY")
				child.RangeGlow:SetInside(child.Health.backdrop)
				child.RangeGlow:SetTexture(E['media'].blankTex)
				child.RangeGlow:SetBlendMode("BLEND")
				child.RangeGlow:SetVertexColor(E.private.rangeModifiers.general.frameColour.r, E.private.rangeModifiers.general.frameColour.g, E.private.rangeModifiers.general.frameColour.b, 0)
				child.RangeGlow:Hide()
			end
		end
	end
	
	if E.private.rangeModifiers.debug then
		print("created " .. childCount .. " RangeGlows")
	end
end

function RangeModifiersModule:UpdateUnitsRange()
	local header, unitsInRange, d, children1, children2, child1, child2, _
	if E.private.rangeModifiers.enabled then
		if ElvUF_Party:IsShown() then
			header = ElvUF_Party
		elseif ElvUF_Raid:IsShown() then
			header = ElvUF_Raid
		elseif ElvUF_Raid40:IsShown() then
			header = ElvUF_Raid40
		end
		
		if header then
			children1 = header:GetChildren()
			for _, child1 in ipairs(children1) do	
				unitsInRange = 0
				
				--child1.RangeGlow:SetVertexColor(E.private.rangeModifiers.general.frameColour.r, E.private.rangeModifiers.general.frameColour.g, E.private.rangeModifiers.general.frameColour.b, 0)
				--child1.RangeGlow:Hide()
				
				children2 = header:GetChildren()
				for _, child2 in ipairs(children2) do
					if child2.unit and UnitIsPlayer(child2.unit) and not UnitIsDead(child2.unit) and UnitIsConnected(child2.unit) then
						d = E:GetDistance(child1.unit, child2.unit) or 0
						if d and d > 0 and d <= E.private.rangeModifiers.general.maxRange then
							unitsInRange = unitsInRange + 1		
						end
					end
				end
				
				unitsInRange = floor(unitsInRange / 2)
				
				if not child1.RangeGlow then
					if E.private.rangeModifiers.debug then
						print("creating RangeGlow on the fly for : " .. child1.unit)
					end
					
					child1.RangeGlow = child1:CreateTexture(nil, "OVERLAY")
					child1.RangeGlow:SetInside(child1.Health.backdrop)
					child1.RangeGlow:SetTexture(E['media'].blankTex)
					child1.RangeGlow:SetBlendMode("BLEND")
				end
						
				if unitsInRange >= E.private.rangeModifiers.general.groupSize then					
					if E.private.rangeModifiers.debug then
						print("enabled: " .. child1.unit .. " has " .. unitsInRange .. " (" .. E.private.rangeModifiers.general.groupSize .. ") units in range")
					end
					if UnitIsDead(child1.unit) then
						child1.RangeGlow:SetVertexColor(E.private.rangeModifiers.general.frameColour.r, E.private.rangeModifiers.general.frameColour.g, E.private.rangeModifiers.general.frameColour.b, 0)
						child1.RangeGlow:Hide()
					else
						child1.RangeGlow:SetVertexColor(E.private.rangeModifiers.general.frameColour.r, E.private.rangeModifiers.general.frameColour.g, E.private.rangeModifiers.general.frameColour.b, alphaValue)
						child1.RangeGlow:Show()
					end
				else
					if E.private.rangeModifiers.debug then
						--print("disabled: " .. child1.unit .. " has " .. unitsInRange .. " units in range")
					end
					child1.RangeGlow:SetVertexColor(E.private.rangeModifiers.general.frameColour.r, E.private.rangeModifiers.general.frameColour.g, E.private.rangeModifiers.general.frameColour.b, 0)
					child1.RangeGlow:Hide()
				end			
			end
		end
	end
end

function RangeModifiersModule:Initialize()
	if not L["RANGEMODIFIERS_LOGIN_MSG2"] or L["RANGEMODIFIERS_LOGIN_MSG2"] == "" then
		print(L["RANGEMODIFIERS_LOGIN_MSG"] .. " " .. L["RANGEMODIFIERS_VERSION"] .. ".")
	else
		print(L["RANGEMODIFIERS_LOGIN_MSG"] .. " " .. L["RANGEMODIFIERS_VERSION"] .. ". " .. L["RANGEMODIFIERS_LOGIN_MSG2"])
	end
	RangeModifiersModule:ToggleEnabled()
end